@@ -9,7 +9,7 @@ use HTTP::Headers;
 use HTTP::Headers::Patch::DontUseStorable -load_target=>0;
 extends 'Finance::Bank::ID::Base';
 
-our $VERSION = '0.26'; # VERSION
+our $VERSION = '0.27'; # VERSION
 
 has _variant => (is => 'rw');
 has _re_tx   => (is => 'rw');
@@ -70,31 +70,40 @@ sub login {
 
     $self->logger->debug('Logging in ...');
     $self->_req(get => [$self->site . "/retail/Login.do?action=form&lang=in_ID"],
-                sub {
-                    my ($mech) = @_;
-                    $mech->content =~ /LoginForm/ or return "no login form";
-                    "";
+                {
+                    id => 'login_form',
+                    after_request => sub {
+                        my ($mech) = @_;
+                        $mech->content =~ /LoginForm/ or return "no login form";
+                        "";
+                    },
                 });
     $self->mech->set_visible(
                              $self->username,
                              $self->password,
                              [image=>"x"]);
     $self->_req(submit => [],
-                sub {
-                    my ($mech) = @_;
-                    $mech->content =~ m!<font class="errorMessage">(.+?)</font>! and return $1;
-                    $mech->content =~ /<frame\s.+Welcome/ and return; # success
-                    $mech->content =~ m!<font class="alert">(\w.+?)</font>! and return $1;
-                    $mech->content =~ /LoginForm/ and
-                        return "submit failed, still getting login form, probably problem with image button";
-                    "unknown login result page";
+                {
+                    id => 'login',
+                    after_request => sub {
+                        my ($mech) = @_;
+                        $mech->content =~ m!<font class="errorMessage">(.+?)</font>! and return $1;
+                        $mech->content =~ /<frame\s.+Welcome/ and return; # success
+                        $mech->content =~ m!<font class="alert">(\w.+?)</font>! and return $1;
+                        $mech->content =~ /LoginForm/ and
+                            return "submit failed, still getting login form, probably problem with image button";
+                        "unknown login result page";
+                    },
                 });
     $self->_req(get => [$self->site . "/retail/Welcome.do?action=result"],
-                sub {
-                    my ($mech) = @_;
-                    $mech->content !~ /SELAMAT DATANG/ and
-                        return "failed getting welcome screen";
-                    "";
+                {
+                    id => 'welcome',
+                    after_request => sub {
+                        my ($mech) = @_;
+                        $mech->content !~ /SELAMAT DATANG/ and
+                            return "failed getting welcome screen";
+                        "";
+                    },
                 });
     $self->logged_in(1);
 }
@@ -104,7 +113,8 @@ sub logout {
 
     return 1 unless $self->logged_in;
     $self->logger->debug('Logging out ...');
-    $self->_req(get => [$self->site . "/retail/Logout.do?action=result"]);
+    $self->_req(get => [$self->site . "/retail/Logout.do?action=result"],
+                {id => 'logout'});
     $self->logged_in(0);
 }
 
@@ -112,7 +122,8 @@ sub _parse_accounts {
     my ($self, $retrieve) = @_;
     $self->login;
     $self->logger->debug("Parsing accounts from transaction history form page ...");
-    $self->_req(get => [$self->site . "/retail/TrxHistoryInq.do?action=form"]) if $retrieve;
+    $self->_req(get => [$self->site . "/retail/TrxHistoryInq.do?action=form"],
+            {id => 'txhist_form-parse_accounts'}) if $retrieve;
     my $ct = $self->mech->content;
     $ct =~ /(HISTORI TRANSAKSI|MUTASI REKENING)/ or
         die "failed getting transaction history form page";
@@ -151,12 +162,15 @@ sub check_balance {
     my $acctid = $self->_get_an_account_id($account, 1);
     my $bal;
     $self->_req(get => ["$s/retail/AccountDetail.do?action=result&ACCOUNTID=$acctid"],
-                sub {
-                    my ($mech) = @_;
-                    $mech->content =~ m!>Informasi Saldo(?:<[^>]+>\s*)*:\s*(?:<[^>]+>\s*)*(?:Rp\.)&nbsp;([0-9.]+),(\d+)<!s
-                        or return "cannot grep balance in result page";
-                    $bal = $self->_stripD($1)+0.01*$2;
-                    "";
+                {
+                    id => "check_balance",
+                    after_request => sub {
+                        my ($mech) = @_;
+                        $mech->content =~ m!>Informasi Saldo(?:<[^>]+>\s*)*:\s*(?:<[^>]+>\s*)*(?:Rp\.)&nbsp;([0-9.]+),(\d+)<!s
+                            or return "cannot grep balance in result page";
+                        $bal = $self->_stripD($1)+0.01*$2;
+                        "";
+                    },
                 });
     $bal;
 }
@@ -169,7 +183,8 @@ sub get_statement {
 
     $self->logger->debug('Getting statement ...');
     my $mech = $self->mech;
-    $self->_req(get => ["$s/retail/TrxHistoryInq.do?action=form"]);
+    $self->_req(get => ["$s/retail/TrxHistoryInq.do?action=form"],
+                {id=>"txhist_form-get_statement"});
 
     my $today = DateTime->today;
     my $end_date = $args{end_date} || $today;
@@ -205,12 +220,15 @@ sub get_statement {
     $mech->set_fields(action => "result");
 
     $self->_req(submit => [],
-                sub {
-                    my ($mech) = @_;
-                    $mech->content =~ /saldo/i and return "";
-                    $mech->content =~ m!<font class="alert">(.+)</font>!
-                        and return $1;
-                    return "failed getting statement";
+                {
+                    id => "get_statement",
+                    after_request => sub {
+                        my ($mech) = @_;
+                        $mech->content =~ /saldo/i and return "";
+                        $mech->content =~ m!<font class="alert">(.+)</font>!
+                            and return $1;
+                        return "failed getting statement";
+                    },
                 });
 
     my $resp = $self->parse_statement($self->mech->content);
@@ -594,18 +612,19 @@ sub _ps_get_transactions_mcm {
 1;
 # ABSTRACT: Check your Bank Mandiri accounts from Perl
 
-
-
 __END__
+
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 Finance::Bank::ID::Mandiri - Check your Bank Mandiri accounts from Perl
 
 =head1 VERSION
 
-version 0.26
+version 0.27
 
 =head1 SYNOPSIS
 
@@ -658,9 +677,6 @@ If you want to use the library in your Perl application:
     # utility routines
     my $res = $ibank->parse_statement($html);
 
-Also see the examples/ subdirectory in the distribution for a sample script
-using this module.
-
 =head1 DESCRIPTION
 
 This module provide a rudimentary interface to the web-based online banking
@@ -874,6 +890,23 @@ Additional notes:
 The method can also (or used to) handle copy-pasted text from the GUI browser,
 but this is no longer documented or guaranteed to keep working.
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Finance-Bank-ID-Mandiri>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/sharyanto/perl-Finance-Bank-ID-Mandiri>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/Public/Dist/Display.html?Name=Finance-Bank-ID-Mandiri
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
@@ -886,4 +919,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
